"""
pycraigslist.tests.test_api.test_query
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Tests instantiation, querying, and query output from pycraigslist .search
and .search_detail methods.
"""

import warnings
from functools import partial

from pytest import mark

import specs
import pycraigslist

# Make a tuple of zipped parameters for `test_query_filters`.
# I.e. pycraigslist.forsale will receive valid filters for the forsale category.
QUERY_FILTERS = tuple(zip(specs.obj.pycraigslist_parents, specs.filters.all_))
# Make a tuple of valid pycraigslist.api.* partial instances for `test_valid_instantiation`.
VALID_INSTANCES = tuple(
    partial(obj, **kwargs)
    for kwargs in specs.kwargs.valid
    for obj in specs.obj.pycraigslist_parents
)


@mark.parametrize("parent", specs.obj.pycraigslist_parents)
def test_dtype_search(parent):
    """Tests output datatype of search method."""
    listings = parent()
    if listings.count > 0:
        post = next(listings.search(limit=1))
        assert isinstance(post, dict)


@mark.parametrize("parent", specs.obj.pycraigslist_parents)
def test_content_search(parent):
    """Tests output content of search method."""
    listings = parent()
    if listings.count > 0:
        post = next(listings.search(limit=1))
        assert all(attr in post.keys() for attr in specs.content.post_content_std.keys())


@mark.parametrize("parent", specs.obj.pycraigslist_parents)
def test_dtype_search_detail(parent):
    """Tests output datatype of search_detail method."""
    listings = parent()
    if listings.count > 0:
        # With include_body=True
        post_detail_w_body = next(listings.search_detail(limit=1, include_body=True))
        assert isinstance(post_detail_w_body, dict)

        # With include_body=False
        post_detail_wo_body = next(listings.search_detail(limit=1, include_body=False))
        assert isinstance(post_detail_wo_body, dict)

        # Without include_body kwarg --> default value is False.
        post_detail = next(listings.search_detail(limit=1))
        assert isinstance(post_detail, dict)


@mark.parametrize("parent", specs.obj.pycraigslist_parents)
def test_content_search_detail(parent):
    """Tests output content of search_detail method."""
    listings = parent()
    if listings.count > 0:
        # With include_body=True
        post_detail_w_body = next(listings.search_detail(limit=1, include_body=True))
        assert all(
            attr in post_detail_w_body.keys()
            for attr in specs.content.post_content_detail_body.keys()
        )

        # With include_body=False
        post_detail_wo_body = next(listings.search_detail(limit=1, include_body=False))
        assert all(
            attr in post_detail_wo_body.keys() for attr in specs.content.post_content_detail.keys()
        )

        # Without include_body kwarg --> default value is False.
        post_detail = next(listings.search_detail(limit=1))
        assert all(attr in post_detail.keys() for attr in specs.content.post_content_detail.keys())


@mark.parametrize(["parent", "filters"], QUERY_FILTERS)
def test_query_filters(parent, filters):
    """Tests search filters plugged into query returns valid pycraigslist
    datatype (i.e. dict obj). Tests every parent category with their respective
    query filters."""
    for filter_ in filters:
        try:
            post = next(parent(filters=filter_).search(limit=10))
            assert isinstance(post, dict)
        except StopIteration:
            # No posts for query and a stop iteration is expected; however, throw warning.
            warnings.warn(
                UserWarning(
                    "encountered StopIteration for filters %s. Adjust if issue persists." % filter_
                )
            )
            assert True


@mark.parametrize("valid_instance", VALID_INSTANCES)
def test_valid_instantiation(valid_instance):
    """Tests proper instantiation of pycraigslist.api.* instances using valid kwargs."""
    assert type(valid_instance()).__bases__[0] == pycraigslist.base.BaseAPI



"""
pycraigslist.tests.specs.static
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Stores static information for testing.
"""

import pycraigslist


class content:
    """Suite of post content templates expected from search and search_detail methods."""

    # Attributes that are guaranteed to exist in standard posts.
    post_content_std = {
        "country": "",
        "region": "",
        "site": "",
        "area": "",
        "category": "",
        "id": "",
        "repost_of": "",
        "last_updated": "",
        "title": "",
        "neighborhood": "",
        "price": "",
        "url": "",
    }

    # Attributes that are guaranteed to exist in detailed posts WITHOUT body.
    post_content_detail = {
        "country": "",
        "region": "",
        "site": "",
        "area": "",
        "category": "",
        "id": "",
        "repost_of": "",
        "last_updated": "",
        "title": "",
        "neighborhood": "",
        "price": "",
        "url": "",
        "lat": "",
        "lon": "",
        "address": "",
        "misc": [],
    }

    # Attributes that are guaranteed to exist in detailed posts WITH body.
    post_content_detail_body = {
        "country": "",
        "region": "",
        "site": "",
        "area": "",
        "category": "",
        "id": "",
        "repost_of": "",
        "last_updated": "",
        "title": "",
        "neighborhood": "",
        "price": "",
        "url": "",
        "lat": "",
        "lon": "",
        "address": "",
        "misc": [],
        "body": "",
    }


class filters:
    """Suite of filters (organized by category) for the `filters` parameter in
    pycraigslist classes."""

    community = (
        {
            "query": "community",
            "posted_today": 1,
        },
        {
            "posted_today": False,
            "bundle_duplicates": 0,
        },
        {
            "search_distance": 10,
            "zip_code": 94538,
        },
    )

    events = (
        {
            "has_image": 1,
            "bundle_duplicates": 1,
        },
        {
            "query": "food",
        },
        {
            "music": 1,
        },
    )

    forsale = (
        {
            "make_model": "toyota",
            "min_price": 3000,
        },
        {
            "min_price": 1000,
            "max_price": 5000,
            "bundle_duplicates": True,
        },
        {
            "zip_code": 94536,
            "has_image": 1,
        },
    )

    gigs = (
        {
            "query": "gardening",
            "posted_today": 1,
            "search_titles": True,
        },
        {
            "search_titles": True,
            "bundle_duplicates": 1,
            "posted_today": 0,
        },
        {
            "has_image": True,
            "posted_today": 0,
            "is_paid": True,
        },
    )

    housing = (
        {
            "min_bedrooms": 1,
            "max_price": 2000,
            "cats_ok": 1,
        },
        {
            "max_price": 1000,
            "dogs_ok": True,
            "has_image": True,
        },
        {
            "min_price": 1200,
            "parking": ["carport", "street parking"],
            "posted_today": True,
        },
    )

    jobs = (
        {
            "query": "software",
            "posted_today": 1,
            "employment_type": ["full-time", "contract"],
        },
        {
            "is_internship": True,
            "posted_today": 0,
        },
        {
            "search_titles": True,
            "bundle_duplicates": 1,
            "query": "restaurant food",
            "employment_type": "part-time",
        },
    )

    resumes = (
        {
            "query": "business",
            "education_level_completed": "high school/GED",
        },
        {
            "has_image": 1,
            "zip_code": 94538,
            "search_distance": 30,
        },
        {
            "posted_today": 1,
            "bundle_duplicates": 1,
        },
    )

    services = (
        {
            "query": "gardening",
            "has_image": True,
        },
        {
            "zip_code": 95060,
            "search_distance": 40,
            "bundle_duplicates": 1,
            "query": "car",
        },
        {
            "has_image": 1,
            "posted_today": 0,
            "bundle_duplicates": 1,
        },
    )

    all_ = (community, events, forsale, gigs, housing, jobs, resumes, services)


class kwargs:
    """Suite of keyword arguments to test intantiation of pycraigslist.api.* object."""

    valid = (
        {"site": "monterey"},
        {"site": "sfbay", "area": "eby"},
        # Use a filter query that is applicable to all queries.
        {"site": "sandiego", "area": "csd", "has_image": True},
        {"site": "sandiego", "area": "csd", "filters": {"has_image": True}},
    )
    invalid = (
        # Invalid site and area
        {"site": "bad_site"},
        {"site": "monterey", "area": "area_not_allowed"},
        {"site": "sfbay", "area": "bad_area"},
        # Invalid filter keys
        {"site": "sandiego", "area": "esd", "bad_filter": "bad_value"},
        {"site": "sandiego", "area": "esd", "filters": {"bad_filter": "bad_value"}},
    )


class limits:
    """Suite of limits for the `limit` parameter in .search and .search_detail methods."""

    std = (2983, 1121, 38, 1928)
    std_fringe = (1, 120, 121, 240, 241)
    detail = (83, 125, 3, 34)
    detail_fringe = (1, 120, 121, 240, 241)


class obj:
    """Suite of pycraigslist objects."""

    pycraigslist_parents = [
        pycraigslist.community,
        pycraigslist.events,
        pycraigslist.forsale,
        pycraigslist.gigs,
        pycraigslist.housing,
        pycraigslist.jobs,
        pycraigslist.resumes,
        pycraigslist.services,
    ]



"""
pycraigslist.api
~~~~~~~~~~~~~~~~

Suite of pycraigslist objects to interface with Craigslist.
"""

from pycraigslist.base import BaseAPI, ParentMethods
from pycraigslist import filters


class community(BaseAPI, ParentMethods):
    """community"""

    category = "ccc"
    query_filters = filters.query.community

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class act(BaseAPI):
        """activity partners"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ats(BaseAPI):
        """artists"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class kid(BaseAPI):
        """childcare"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class com(BaseAPI):
        """general community"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class grp(BaseAPI):
        """groups"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vnn(BaseAPI):
        """local news and views"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class laf(BaseAPI):
        """lost & found"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mis(BaseAPI):
        """missed connections"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class muc(BaseAPI):
        """musicians"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pet(BaseAPI):
        """pets"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pol(BaseAPI):
        """politics"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rnr(BaseAPI):
        """rants & raves"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rid(BaseAPI):
        """rideshare"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vol(BaseAPI):
        """volunteers"""

        query_filters = filters.query.community

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class events(BaseAPI, ParentMethods):
    """events"""

    category = "eee"
    query_filters = filters.query.events

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class forsale(BaseAPI, ParentMethods):
    """for sale"""

    category = "sss"
    query_filters = filters.query.forsale

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class ata(BaseAPI):
        """antiques"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ppa(BaseAPI):
        """appliances"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ppd(BaseAPI):
        """appliances - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class app(BaseAPI):
        """appliances - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ara(BaseAPI):
        """arts & crafts"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ard(BaseAPI):
        """arts & crafts - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class art(BaseAPI):
        """arts & crafts - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sna(BaseAPI):
        """atvs, utvs, snowmobiles"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class snd(BaseAPI):
        """atvs, utvs, snowmobiles - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class snw(BaseAPI):
        """atvs, utvs, snowmobiles - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pta(BaseAPI):
        """auto parts"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ptd(BaseAPI):
        """auto parts - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pts(BaseAPI):
        """auto parts - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wta(BaseAPI):
        """auto wheels & tires"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wtd(BaseAPI):
        """auto wheels & tires - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wto(BaseAPI):
        """auto wheels & tires - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ava(BaseAPI):
        """aviation"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class avd(BaseAPI):
        """aviation - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class avo(BaseAPI):
        """aviation - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class baa(BaseAPI):
        """baby & kid stuff"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bad(BaseAPI):
        """baby & kid stuff - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bab(BaseAPI):
        """baby & kid stuff - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bar(BaseAPI):
        """barter"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bip(BaseAPI):
        """bicycle parts"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bdp(BaseAPI):
        """bicycle parts - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bop(BaseAPI):
        """bicycle parts - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bia(BaseAPI):
        """bicycles"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bid(BaseAPI):
        """bicycles - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bik(BaseAPI):
        """bicycles - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bpa(BaseAPI):
        """boat parts & accessories"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bpd(BaseAPI):
        """boat parts & accessories - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bpo(BaseAPI):
        """boat parts & accessories - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class boo(BaseAPI):
        """boats"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bod(BaseAPI):
        """boats - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class boa(BaseAPI):
        """boats - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bka(BaseAPI):
        """books & magazines"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bkd(BaseAPI):
        """books & magazines - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bks(BaseAPI):
        """books & magazines - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bfa(BaseAPI):
        """business"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bfd(BaseAPI):
        """business - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bfs(BaseAPI):
        """business - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cta(BaseAPI):
        """cars & trucks"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ctd(BaseAPI):
        """cars & trucks - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cto(BaseAPI):
        """cars & trucks - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ema(BaseAPI):
        """cds / dvds / vhs"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class emq(BaseAPI):
        """cds / dvds / vhs - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class emd(BaseAPI):
        """cds / dvds / vhs - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class moa(BaseAPI):
        """cell phones"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mod(BaseAPI):
        """cell phones - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mob(BaseAPI):
        """cell phones - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cla(BaseAPI):
        """clothing & accessories"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cld(BaseAPI):
        """clothing & accessories - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class clo(BaseAPI):
        """clothing & accessories - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cba(BaseAPI):
        """collectibles"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cbd(BaseAPI):
        """collectibles - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class clt(BaseAPI):
        """collectibles - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class syp(BaseAPI):
        """computer parts"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sdp(BaseAPI):
        """computer parts - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sop(BaseAPI):
        """computer parts - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sya(BaseAPI):
        """computers"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class syd(BaseAPI):
        """computers - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sys(BaseAPI):
        """computers - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ela(BaseAPI):
        """electronics"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class eld(BaseAPI):
        """electronics - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ele(BaseAPI):
        """electronics - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class gra(BaseAPI):
        """farm & garden"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class zip(BaseAPI):
        """free stuff"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fua(BaseAPI):
        """furniture"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class gms(BaseAPI):
        """garage & moving sales"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class foa(BaseAPI):
        """general for sale"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fod(BaseAPI):
        """general for sale - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fur(BaseAPI):
        """general for sale - by owner"""

        # An odd-ball - give 'for' alias 'fur'.

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = "for"
            super().__init__(*args, **kwargs)

    class haa(BaseAPI):
        """health and beauty"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class had(BaseAPI):
        """health and beauty - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hab(BaseAPI):
        """health and beauty - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hva(BaseAPI):
        """heavy equipment"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hvd(BaseAPI):
        """heavy equipment - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hvo(BaseAPI):
        """heavy equipment - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hsa(BaseAPI):
        """household items"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hsd(BaseAPI):
        """household items - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hsh(BaseAPI):
        """household items - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class jwa(BaseAPI):
        """jewelry"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class jwd(BaseAPI):
        """jewelry - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class jwl(BaseAPI):
        """jewelry - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class maa(BaseAPI):
        """materials"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mad(BaseAPI):
        """materials - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mat(BaseAPI):
        """materials - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mpa(BaseAPI):
        """motorcycle parts & accessories"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mpd(BaseAPI):
        """motorcycle parts & accessories - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mpo(BaseAPI):
        """motorcycle parts & accessories - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mca(BaseAPI):
        """motorcycles/scooters"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mcd(BaseAPI):
        """motorcycles/scooters - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mcy(BaseAPI):
        """motorcycles/scooters - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class msa(BaseAPI):
        """musical instruments"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class msd(BaseAPI):
        """musical instruments - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class msg(BaseAPI):
        """musical instruments - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pha(BaseAPI):
        """photo/video"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class phd(BaseAPI):
        """photo/video - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pho(BaseAPI):
        """photo/video - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rva(BaseAPI):
        """recreational vehicles"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rvd(BaseAPI):
        """recreational vehicles - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rvs(BaseAPI):
        """recreational vehicles - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sga(BaseAPI):
        """sporting goods"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sgd(BaseAPI):
        """sporting goods - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class spo(BaseAPI):
        """sporting goods - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tia(BaseAPI):
        """tickets"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tid(BaseAPI):
        """tickets - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tix(BaseAPI):
        """tickets - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tla(BaseAPI):
        """tools"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class taa(BaseAPI):
        """toys & games"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tad(BaseAPI):
        """toys & games - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tag(BaseAPI):
        """toys & games - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tra(BaseAPI):
        """trailers"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class trb(BaseAPI):
        """trailers - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tro(BaseAPI):
        """trailers - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vga(BaseAPI):
        """video gaming"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vgd(BaseAPI):
        """video gaming - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vgm(BaseAPI):
        """video gaming - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class waa(BaseAPI):
        """wanted"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wad(BaseAPI):
        """wanted - by dealer"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wan(BaseAPI):
        """wanted - by owner"""

        query_filters = filters.query.forsale

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class gigs(BaseAPI, ParentMethods):
    """gigs"""

    category = "ggg"
    query_filters = filters.query.gigs

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class cpg(BaseAPI):
        """computer gigs"""

        query_filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class crg(BaseAPI):
        """creative gigs"""

        query_filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cwg(BaseAPI):
        """crew gigs"""

        query_filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class dmg(BaseAPI):
        """domestic gigs"""

        query_filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class evg(BaseAPI):
        """event gigs"""

        query_filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lbg(BaseAPI):
        """labor gigs"""

        query_filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tlg(BaseAPI):
        """talent gigs"""

        query_filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wrg(BaseAPI):
        """writing gigs"""

        query_filters = filters.query.gigs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class housing(BaseAPI, ParentMethods):
    """housing"""

    category = "hhh"
    query_filters = filters.query.housing

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class apa(BaseAPI):
        """apartments / housing for rent"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class swp(BaseAPI):
        """housing swap"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class off(BaseAPI):
        """office & commercial"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class prk(BaseAPI):
        """parking & storage"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rea(BaseAPI):
        """real estate"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class reb(BaseAPI):
        """real estate - by dealer"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class reo(BaseAPI):
        """real estate - by owner"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class roo(BaseAPI):
        """rooms & shares"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sub(BaseAPI):
        """sublets & temporary"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class vac(BaseAPI):
        """vacation rentals"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hou(BaseAPI):
        """wanted: apts"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rew(BaseAPI):
        """wanted: real estate"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sha(BaseAPI):
        """wanted: room/share"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sbw(BaseAPI):
        """wanted: sublet/temp"""

        query_filters = filters.query.housing

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class jobs(BaseAPI, ParentMethods):
    """jobs"""

    category = "jjj"
    query_filters = filters.query.jobs

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class acc(BaseAPI):
        """accounting/finance"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ofc(BaseAPI):
        """admin/office"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class egr(BaseAPI):
        """architect/engineer/cad"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class med(BaseAPI):
        """art/media/design"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bus(BaseAPI):
        """business/mgmt"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class csr(BaseAPI):
        """customer service"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class edu(BaseAPI):
        """education/teaching"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class etc(BaseAPI):
        """et cetera"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fbh(BaseAPI):
        """food/beverage/hospitality"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lab(BaseAPI):
        """general labor"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class gov(BaseAPI):
        """government"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hea(BaseAPI):
        """healthcare"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hum(BaseAPI):
        """human resource"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lgl(BaseAPI):
        """legal/paralegal"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mnu(BaseAPI):
        """manufacturing"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mar(BaseAPI):
        """marketing/advertising/pr"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class npo(BaseAPI):
        """nonprofit"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rej(BaseAPI):
        """real estate"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class ret(BaseAPI):
        """retail/wholesale"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sls(BaseAPI):
        """sales"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class spa(BaseAPI):
        """salon/spa/fitness"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sci(BaseAPI):
        """science/biotech"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sec(BaseAPI):
        """security"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class trd(BaseAPI):
        """skilled trades/artisan"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sof(BaseAPI):
        """software/qa/dba/etc"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sad(BaseAPI):
        """systems/networking"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tch(BaseAPI):
        """technical support"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class trp(BaseAPI):
        """transportation"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class tfr(BaseAPI):
        """tv/film/video/radio"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class web(BaseAPI):
        """web/html/info design"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wri(BaseAPI):
        """writing/editing"""

        query_filters = filters.query.jobs

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)


class resumes(BaseAPI, ParentMethods):
    """resumes"""

    category = "rrr"
    query_filters = filters.query.resumes

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class services(BaseAPI, ParentMethods):
    """services"""

    category = "bbb"
    query_filters = filters.query.services

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    class aos(BaseAPI):
        """automotive services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class bts(BaseAPI):
        """beauty services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cms(BaseAPI):
        """cell phone / mobile services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cps(BaseAPI):
        """computer services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class crs(BaseAPI):
        """creative services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class cys(BaseAPI):
        """cycle services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class evs(BaseAPI):
        """event services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fgs(BaseAPI):
        """farm & garden services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class fns(BaseAPI):
        """financial services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class hss(BaseAPI):
        """household services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lbs(BaseAPI):
        """labor / hauling / moving"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lgs(BaseAPI):
        """legal services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class lss(BaseAPI):
        """lessons & tutoring"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class mas(BaseAPI):
        """marine services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class pas(BaseAPI):
        """pet services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class rts(BaseAPI):
        """real estate services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class sks(BaseAPI):
        """skilled trade services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class biz(BaseAPI):
        """small biz ads"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class trv(BaseAPI):
        """travel/vacation services"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)

    class wet(BaseAPI):
        """writing / editing / translation"""

        query_filters = filters.query.services

        def __init__(self, *args, **kwargs):
            self.category = self.__class__.__name__
            super().__init__(*args, **kwargs)



"""
pycraigslist.filters
~~~~~~~~~~~~~~~~~~~~

Stores Craigslist query filters, categories, and regions.
"""


class query:
    """Store query filters by broad category."""

    base = {
        "query": {"url_key": "query", "value": None},
        "search_titles": {"url_key": "srchType", "value": "T"},
        "has_image": {"url_key": "hasPic", "value": 1},
        "posted_today": {"url_key": "postedToday", "value": 1},
        "bundle_duplicates": {"url_key": "bundleDuplicates", "value": 1},
        "search_distance": {"url_key": "search_distance", "value": None},
        "zip_code": {"url_key": "postal", "value": None},
    }

    community = {
        **base,
        **{
            # No unique filters for community.
        },
    }

    events = {
        **base,
        **{
            # art/film
            "art": {"url_key": "event_art", "value": 1, "attr": "art/film"},
            "film": {"url_key": "event_art", "value": 1, "attr": "art/film"},
            # career
            "career": {"url_key": "event_career", "value": 1, "attr": "career"},
            # charitable
            "charitable": {"url_key": "event_fundraiser_vol", "value": 1, "attr": "charitable"},
            "fundraiser": {"url_key": "event_fundraiser_vol", "value": 1, "attr": "charitable"},
            # competiton
            "athletics": {"url_key": "event_athletics", "value": 1, "attr": "competition"},
            "competition": {"url_key": "event_athletics", "value": 1, "attr": "competition"},
            # dance
            "dance": {"url_key": "event_dance", "value": 1, "attr": "dance"},
            # fest/fair
            "festival": {"url_key": "event_festival", "value": 1, "attr": "fest/fair"},
            "fair": {"url_key": "event_festival", "value": 1, "attr": "fest/fair"},
            # fitness/health
            "fitness": {"url_key": "event_fitness_wellness", "value": 1, "attr": "fitness/health"},
            "health": {"url_key": "event_fitness_wellness", "value": 1, "attr": "fitness/health"},
            # food/drink
            "food": {"url_key": "event_food", "value": 1, "attr": "food/drink"},
            "drink": {"url_key": "event_food", "value": 1, "attr": "food/drink"},
            # free
            "free": {"url_key": "event_free", "value": 1, "attr": "free"},
            # kid friendly
            "kid_friendly": {"url_key": "event_kidfriendly", "value": 1, "attr": "kid friendly"},
            # literary
            "literary": {"url_key": "event_literary", "value": 1, "attr": "literary"},
            # music
            "music": {"url_key": "event_music", "value": 1, "attr": "music"},
            # outdoor
            "outdoor": {"url_key": "event_outdoor", "value": 1, "attr": "outdoor"},
            # sale
            "sale": {"url_key": "event_sale", "value": 1, "attr": "sale"},
            # singles
            "singles": {"url_key": "event_singles", "value": 1, "attr": "singles"},
            # tech
            "tech": {"url_key": "event_geek", "value": 1, "attr": "tech"},
        },
    }

    forsale = {
        **base,
        **{
            # price
            "min_price": {"url_key": "min_price", "value": None},
            "max_price": {"url_key": "max_price", "value": None},
            # make and model
            "make_model": {"url_key": "auto_make_model", "value": None},
            "min_year": {"url_key": "min_auto_year", "value": None},
            "max_year": {"url_key": "max_auto_year", "value": None},
            # odometer
            "min_miles": {"url_key": "min_auto_miles", "value": None},
            "max_miles": {"url_key": "max_auto_miles", "value": None},
            # engine displacement (cc)
            "min_engine_displacement": {"url_key": "min_engine_displacement_cc", "value": None},
            "max_engine_displacement": {"url_key": "max_engine_displacement_cc", "value": None},
        },
    }

    gigs = {
        **base,
        **{
            # paid/unpaid
            "is_paid": {"url_key": "is_paid", "value": 1},
        },
    }

    housing = {
        **base,
        **{
            # price
            "min_price": {"url_key": "min_price", "value": None},
            "max_price": {"url_key": "max_price", "value": None},
            # bedrooms
            "min_bedrooms": {"url_key": "min_bedrooms", "value": None},
            "max_bedrooms": {"url_key": "max_bedrooms", "value": None},
            # bathrooms
            "min_bathrooms": {"url_key": "min_bathrooms", "value": None},
            "max_bathrooms": {"url_key": "max_bathrooms", "value": None},
            # ft2
            "min_ft2": {"url_key": "minSqft", "value": None},
            "max_ft2": {"url_key": "maxSqft", "value": None},
            # private room
            "private_room": {"url_key": "private_room", "value": 1, "attr": "private room"},
            # private bath
            "private_bath": {"url_key": "private_bath", "value": 1, "attr": "private bath"},
            # cats ok
            "cats_ok": {"url_key": "pets_cat", "value": 1, "attr": "cats are ok - purrr"},
            # dogs ok
            "dogs_ok": {"url_key": "pets_dog", "value": 1, "attr": "dogs are ok - wooof"},
            # furnished
            "is_furnished": {"url_key": "is_furnished", "value": 1, "attr": "furnished"},
            # no smoking
            "no_smoking": {"url_key": "no_smoking", "value": 1, "attr": "no smoking"},
            # wheelchair access
            "wheelchair_acccess": {
                "url_key": "wheelchaccess",
                "value": 1,
                "attr": "wheelchair accessible",
            },
            # EV charging
            "ev_charging": {"url_key": "ev_charging", "value": 1, "attr": "ev charging"},
            # no appliation fee
            "no_application_fee": {"url_key": "application_fee", "value": 1},
            # no broker fee
            "no_broker_fee": {"url_key": "broker_fee", "value": 1},
        },
    }

    jobs = {
        **base,
        **{
            # internship
            "is_internship": {"url_key": "is_internship", "value": 1, "attr": "internship"},
            # non-profit
            "is_nonprofit": {
                "url_key": "is_nonprofit",
                "value": 1,
                "attr": "non-profit organization",
            },
            # telecommute
            "is_telecommuting": {
                "url_key": "is_telecommuting",
                "value": 1,
                "attr": "telecommuting okay",
            },
        },
    }

    resumes = {
        **base,
        **{
            # TODO: Please create an issue or PR if interested in this category.
        },
    }

    services = {
        **base,
        **{
            # No unique filters for services.
        },
    }


class category:
    """Store niche category keys and descriptions within broad category."""

    @classmethod
    def get(cls, key):
        """Gets niche categories from broad category key."""
        index_table = {
            "community": cls.community,
            "events": cls.events,
            "forsale": cls.forsale,
            "gigs": cls.gigs,
            "housing": cls.housing,
            "jobs": cls.jobs,
            "resumes": cls.resumes,
            "services": cls.services,
        }
        return index_table[key]

    community = {
        "act": "activity partners",
        "ats": "artists",
        "kid": "childcare",
        "com": "general community",
        "grp": "groups",
        "vnn": "local news and views",
        "laf": "lost & found",
        "mis": "missed connections",
        "muc": "musicians",
        "pet": "pets",
        "pol": "politics",
        "rnr": "rants & raves",
        "rid": "rideshare",
        "vol": "volunteers",
    }

    events = {}

    forsale = {
        "ata": "antiques",
        "ppa": "appliances",
        "ppd": "appliances - by dealer",
        "app": "appliances - by owner",
        "ara": "arts & crafts",
        "ard": "arts & crafts - by dealer",
        "art": "arts & crafts - by owner",
        "sna": "atvs, utvs, snowmobiles",
        "snd": "atvs, utvs, snowmobiles - by dealer",
        "snw": "atvs, utvs, snowmobiles - by owner",
        "pta": "auto parts",
        "ptd": "auto parts - by dealer",
        "pts": "auto parts - by owner",
        "wta": "auto wheels & tires",
        "wtd": "auto wheels & tires - by dealer",
        "wto": "auto wheels & tires - by owner",
        "ava": "aviation",
        "avd": "aviation - by dealer",
        "avo": "aviation - by owner",
        "baa": "baby & kid stuff",
        "bad": "baby & kid stuff - by dealer",
        "bab": "baby & kid stuff - by owner",
        "bar": "barter",
        "bip": "bicycle parts",
        "bdp": "bicycle parts - by dealer",
        "bop": "bicycle parts - by owner",
        "bia": "bicycles",
        "bid": "bicycles - by dealer",
        "bik": "bicycles - by owner",
        "bpa": "boat parts & accessories",
        "bpd": "boat parts & accessories - by dealer",
        "bpo": "boat parts & accessories - by owner",
        "boo": "boats",
        "bod": "boats - by dealer",
        "boa": "boats - by owner",
        "bka": "books & magazines",
        "bkd": "books & magazines - by dealer",
        "bks": "books & magazines - by owner",
        "bfa": "business",
        "bfd": "business - by dealer",
        "bfs": "business - by owner",
        "cta": "cars & trucks",
        "ctd": "cars & trucks - by dealer",
        "cto": "cars & trucks - by owner",
        "ema": "cds / dvds / vhs",
        "emq": "cds / dvds / vhs - by dealer",
        "emd": "cds / dvds / vhs - by owner",
        "moa": "cell phones",
        "mod": "cell phones - by dealer",
        "mob": "cell phones - by owner",
        "cla": "clothing & accessories",
        "cld": "clothing & accessories - by dealer",
        "clo": "clothing & accessories - by owner",
        "cba": "collectibles",
        "cbd": "collectibles - by dealer",
        "clt": "collectibles - by owner",
        "syp": "computer parts",
        "sdp": "computer parts - by dealer",
        "sop": "computer parts - by owner",
        "sya": "computers",
        "syd": "computers - by dealer",
        "sys": "computers - by owner",
        "ela": "electronics",
        "eld": "electronics - by dealer",
        "ele": "electronics - by owner",
        "gra": "farm & garden",
        "zip": "free stuff",
        "fua": "furniture",
        "gms": "garage & moving sales",
        "foa": "general for sale",
        "fod": "general for sale - by dealer",
        "fur": "general for sale - by owner",
        "haa": "health and beauty",
        "had": "health and beauty - by dealer",
        "hab": "health and beauty - by owner",
        "hva": "heavy equipment",
        "hvd": "heavy equipment - by dealer",
        "hvo": "heavy equipment - by owner",
        "hsa": "household items",
        "hsd": "household items - by dealer",
        "hsh": "household items - by owner",
        "jwa": "jewelry",
        "jwd": "jewelry - by dealer",
        "jwl": "jewelry - by owner",
        "maa": "materials",
        "mad": "materials - by dealer",
        "mat": "materials - by owner",
        "mpa": "motorcycle parts & accessories",
        "mpd": "motorcycle parts & accessories - by dealer",
        "mpo": "motorcycle parts & accessories - by owner",
        "mca": "motorcycles/scooters",
        "mcd": "motorcycles/scooters - by dealer",
        "mcy": "motorcycles/scooters - by owner",
        "msa": "musical instruments",
        "msd": "musical instruments - by dealer",
        "msg": "musical instruments - by owner",
        "pha": "photo/video",
        "phd": "photo/video - by dealer",
        "pho": "photo/video - by owner",
        "rva": "recreational vehicles",
        "rvd": "recreational vehicles - by dealer",
        "rvs": "recreational vehicles - by owner",
        "sga": "sporting goods",
        "sgd": "sporting goods - by dealer",
        "spo": "sporting goods - by owner",
        "tia": "tickets",
        "tid": "tickets - by dealer",
        "tix": "tickets - by owner",
        "tla": "tools",
        "taa": "toys & games",
        "tad": "toys & games - by dealer",
        "tag": "toys & games - by owner",
        "tra": "trailers",
        "trb": "trailers - by dealer",
        "tro": "trailers - by owner",
        "vga": "video gaming",
        "vgd": "video gaming - by dealer",
        "vgm": "video gaming - by owner",
        "waa": "wanted",
        "wad": "wanted - by dealer",
        "wan": "wanted - by owner",
    }

    gigs = {
        "cpg": "computer gigs",
        "crg": "creative gigs",
        "cwg": "crew gigs",
        "dmg": "domestic gigs",
        "evg": "event gigs",
        "lbg": "labor gigs",
        "tlg": "talent gigs",
        "wrg": "writing gigs",
    }

    housing = {
        "apa": "apartments / housing for rent",
        "swp": "housing swap",
        "off": "office & commercial",
        "prk": "parking & storage",
        "rea": "real estate",
        "reb": "real estate - by dealer",
        "reo": "real estate - by owner",
        "roo": "rooms & shares",
        "sub": "sublets & temporary",
        "vac": "vacation rentals",
        "hou": "wanted: apts",
        "rew": "wanted: real estate",
        "sha": "wanted: room/share",
        "sbw": "wanted: sublet/temp",
    }

    jobs = {
        "acc": "accounting/finance",
        "ofc": "admin/office",
        "egr": "architect/engineer/cad",
        "med": "art/media/design",
        "bus": "business/mgmt",
        "csr": "customer service",
        "edu": "education/teaching",
        "etc": "et cetera",
        "fbh": "food/beverage/hospitality",
        "lab": "general labor",
        "gov": "government",
        "hea": "healthcare",
        "hum": "human resource",
        "lgl": "legal/paralegal",
        "mnu": "manufacturing",
        "mar": "marketing/advertising/pr",
        "npo": "nonprofit",
        "rej": "real estate",
        "ret": "retail/wholesale",
        "sls": "sales",
        "spa": "salon/spa/fitness",
        "sci": "science/biotech",
        "sec": "security",
        "trd": "skilled trades/artisan",
        "sof": "software/qa/dba/etc",
        "sad": "systems/networking",
        "tch": "technical support",
        "trp": "transportation",
        "tfr": "tv/film/video/radio",
        "web": "web/html/info design",
        "wri": "writing/editing",
    }

    resumes = {}

    services = {
        "aos": "automotive services",
        "bts": "beauty services",
        "cms": "cell phone / mobile services",
        "cps": "computer services",
        "crs": "creative services",
        "cys": "cycle services",
        "evs": "event services",
        "fgs": "farm & garden services",
        "fns": "financial services",
        "hss": "household services",
        "lbs": "labor / hauling / moving",
        "lgs": "legal services",
        "lss": "lessons & tutoring",
        "mas": "marine services",
        "pas": "pet services",
        "rts": "real estate services",
        "sks": "skilled trade services",
        "biz": "small biz ads",
        "trv": "travel/vacation services",
        "wet": "writing / editing / translation",
    }


class region:
    """Store valid Craigslist site and area keys. Last updated: 2021-08-08."""

    areas = {
        "atlanta": {"nat", "wat", "eat", "sat", "atl"},
        "boston": {"gbs", "bmw", "sob", "nos", "nwb"},
        "chicago": {"chc", "nwc", "nch", "sox", "nwi", "wcl"},
        "dallas": {"dal", "sdf", "mdf", "ndf", "ftw"},
        "detroit": {"wyn", "mcb", "okl"},
        "fortlauderdale": {"brw", "mdc", "pbc"},
        "fortmyers": {"col", "lee", "chl"},
        "losangeles": {"ant", "sgv", "lac", "lgb", "sfv", "wst"},
        "miami": {"brw", "mdc"},
        "minneapolis": {"wsh", "ank", "hnp", "csw", "dak", "ram"},
        "newyork": {"brk", "mnh", "stn", "wch", "lgi", "fct", "brx", "jsy", "que"},
        "phoenix": {"evl", "nph", "wvl", "cph"},
        "portland": {"mlt", "clc", "yam", "nco", "clk", "grg", "wsc"},
        "sandiego": {"ssd", "csd", "esd", "nsd"},
        "seattle": {"sno", "oly", "see", "kit", "skc", "est", "tac"},
        "sfbay": {"eby", "scz", "pen", "sby", "sfc", "nby"},
        "tampa": {"psc", "hil", "pnl", "hdo"},
        "toronto": {"mss", "yrk", "drh", "oak", "bra", "tor"},
        "vancouver": {"nvn", "pml", "van", "rch", "bnc", "rds"},
    }

    # fmt: off
    sites = {
        'abbotsford', 'aberdeen', 'abilene', 'acapulco', 'accra', 'addisababa',
        'adelaide', 'ahmedabad', 'akroncanton', 'albany', 'albanyga', 'albuquerque',
        'alicante', 'allentown', 'altoona', 'amarillo', 'ames', 'amsterdam',
        'anchorage', 'annapolis', 'annarbor', 'appleton', 'asheville', 'ashtabula',
        'athens', 'athensga', 'athensohio', 'atlanta', 'auburn', 'auckland',
        'austin', 'bacolod', 'baghdad', 'bajasur', 'bakersfield', 'baleares',
        'baltimore', 'bangalore', 'bangkok', 'bangladesh', 'barcelona', 'barrie',
        'basel', 'bath', 'batonrouge', 'battlecreek', 'beaumont', 'beijing',
        'beirut', 'belfast', 'belleville', 'bellingham', 'belohorizonte', 'bemidji',
        'bend', 'berlin', 'bern', 'bgky', 'bham', 'bhubaneswar',
        'bigbend', 'bilbao', 'billings', 'binghamton', 'birmingham', 'bismarck',
        'blacksburg', 'bloomington', 'bn', 'boise', 'bologna', 'boone',
        'bordeaux', 'boston', 'boulder', 'bozeman', 'brainerd', 'brantford',
        'brasilia', 'bremen', 'brighton', 'brisbane', 'bristol', 'brownsville',
        'brunswick', 'brussels', 'bucharest', 'budapest', 'buenosaires', 'buffalo',
        'bulgaria', 'butte', 'cadiz', 'cairns', 'cairo', 'calgary',
        'cambridge', 'canarias', 'canberra', 'capetown', 'caracas', 'carbondale',
        'cardiff', 'cariboo', 'casablanca', 'catskills', 'cdo', 'cebu',
        'cedarrapids', 'cenla', 'centralmich', 'cfl', 'chambana', 'chambersburg',
        'chandigarh', 'charleston', 'charlestonwv', 'charlotte', 'charlottesville', 'chatham',
        'chattanooga', 'chautauqua', 'chengdu', 'chennai', 'chicago', 'chico',
        'chihuahua', 'chillicothe', 'chongqing', 'christchurch', 'cincinnati', 'clarksville',
        'cleveland', 'clovis', 'cnj', 'collegestation', 'cologne', 'colombia',
        'columbia', 'columbiamo', 'columbus', 'columbusga', 'comoxvalley', 'cookeville',
        'copenhagen', 'cornwall', 'corpuschristi', 'corvallis', 'cosprings', 'costarica',
        'cotedazur', 'coventry', 'csd', 'curitiba', 'dalian', 'dallas',
        'danville', 'darwin', 'davaocity', 'dayton', 'daytona', 'delaware',
        'delhi', 'denver', 'derby', 'desmoines', 'detroit', 'devon',
        'dothan', 'dresden', 'dubai', 'dublin', 'dubuque', 'duluth',
        'dundee', 'dunedin', 'durban', 'dusseldorf', 'eastco', 'easternshore',
        'eastidaho', 'eastky', 'eastmids', 'eastnc', 'eastoregon', 'easttexas',
        'eauclaire', 'edinburgh', 'edmonton', 'elko', 'elmira', 'elpaso',
        'elsalvador', 'enid', 'erie', 'essen', 'essex', 'eugene',
        'evansville', 'fairbanks', 'fargo', 'farmington', 'faro', 'fayar',
        'fayetteville', 'fingerlakes', 'flagstaff', 'florence', 'florencesc', 'fortaleza',
        'fortcollins', 'fortdodge', 'fortlauderdale', 'fortmyers', 'fortsmith', 'fortwayne',
        'frankfurt', 'frederick', 'fredericksburg', 'fresno', 'ftmcmurray', 'fukuoka',
        'gadsden', 'galveston', 'geneva', 'genoa', 'glasgow', 'glensfalls',
        'goa', 'goldcoast', 'goldcountry', 'granada', 'grandforks', 'grandisland',
        'grandrapids', 'greatfalls', 'greenbay', 'greensboro', 'greenville', 'grenoble',
        'guadalajara', 'guanajuato', 'guangzhou', 'guatemala', 'guelph', 'gulfport',
        'haifa', 'halifax', 'hamburg', 'hamilton', 'hampshire', 'hanford',
        'hangzhou', 'hannover', 'harrisburg', 'harrisonburg', 'hartford', 'hat',
        'hattiesburg', 'heidelberg', 'helena', 'helsinki', 'hermosillo', 'hickory',
        'hiltonhead', 'hiroshima', 'hobart', 'holland', 'hongkong', 'honolulu',
        'houma', 'houston', 'hudsonvalley', 'humboldt', 'huntington', 'huntsville',
        'hyderabad', 'iloilo', 'imperial', 'indianapolis', 'indore', 'inlandempire',
        'iowacity', 'istanbul', 'ithaca', 'jackson', 'jacksontn', 'jacksonville',
        'jaipur', 'jakarta', 'janesville', 'jerseyshore', 'jerusalem', 'johannesburg',
        'jonesboro', 'joplin', 'juarez', 'juneau', 'jxn', 'kaiserslautern',
        'kalamazoo', 'kalispell', 'kamloops', 'kansascity', 'kelowna', 'kenai',
        'kent', 'kenya', 'kerala', 'keys', 'killeen', 'kingston',
        'kirksville', 'kitchener', 'klamath', 'knoxville', 'kokomo', 'kolkata',
        'kootenays', 'kpr', 'ksu', 'kuwait', 'lacrosse', 'lafayette',
        'lakecharles', 'lakecity', 'lakeland', 'lancaster', 'lansing', 'lapaz',
        'laredo', 'lasalle', 'lascruces', 'lasvegas', 'lausanne', 'lawrence',
        'lawton', 'leeds', 'leipzig', 'lethbridge', 'lewiston', 'lexington',
        'lille', 'lima', 'limaohio', 'lincoln', 'lisbon', 'littlerock',
        'liverpool', 'logan', 'loire', 'london', 'londonon', 'longisland',
        'losangeles', 'louisville', 'loz', 'lubbock', 'lucknow', 'luxembourg',
        'lynchburg', 'lyon', 'macon', 'madison', 'madrid', 'maine',
        'malaga', 'malaysia', 'managua', 'manchester', 'manila', 'mankato',
        'mansfield', 'marseilles', 'marshall', 'martinsburg', 'masoncity', 'mattoon',
        'mazatlan', 'mcallen', 'meadville', 'medford', 'melbourne', 'memphis',
        'merced', 'meridian', 'mexicocity', 'miami', 'micronesia', 'milan',
        'milwaukee', 'minneapolis', 'missoula', 'mobile', 'modesto', 'mohave',
        'monroe', 'monroemi', 'montana', 'monterey', 'monterrey', 'montevideo',
        'montgomery', 'montpellier', 'montreal', 'morgantown', 'moscow', 'moseslake',
        'mumbai', 'muncie', 'munich', 'muskegon', 'myrtlebeach', 'nacogdoches',
        'naga', 'nagoya', 'nanaimo', 'nanjing', 'naples', 'nashville',
        'natchez', 'nd', 'nesd', 'newbrunswick', 'newcastle', 'newfoundland',
        'newhaven', 'newjersey', 'newlondon', 'neworleans', 'newyork', 'nh',
        'niagara', 'nmi', 'norfolk', 'northernwi', 'northmiss', 'northplatte',
        'norwich', 'nottingham', 'ntl', 'nuremberg', 'nwct', 'nwga',
        'nwks', 'oaxaca', 'odessa', 'ogden', 'okaloosa', 'okinawa',
        'oklahomacity', 'olympic', 'omaha', 'onslow', 'orangecounty', 'oregoncoast',
        'orlando', 'osaka', 'oslo', 'ottawa', 'ottumwa', 'outerbanks',
        'owensboro', 'owensound', 'oxford', 'pakistan', 'palmsprings', 'pampanga',
        'panama', 'panamacity', 'paris', 'parkersburg', 'peace', 'pei',
        'pennstate', 'pensacola', 'peoria', 'perth', 'perugia', 'peterborough',
        'philadelphia', 'phoenix', 'pittsburgh', 'plattsburgh', 'poconos', 'porthuron',
        'portland', 'porto', 'portoalegre', 'potsdam', 'prague', 'pretoria',
        'princegeorge', 'providence', 'provo', 'puebla', 'pueblo', 'puertorico',
        'pullman', 'pune', 'pv', 'quadcities', 'quebec', 'quincy',
        'quito', 'racine', 'raleigh', 'ramallah', 'rapidcity', 'reading',
        'recife', 'reddeer', 'redding', 'regina', 'rennes', 'reno',
        'reykjavik', 'richmond', 'richmondin', 'rio', 'roanoke', 'rochester',
        'rockford', 'rockies', 'rome', 'roswell', 'rouen', 'sacramento',
        'saginaw', 'saguenay', 'salem', 'salina', 'saltlakecity', 'salvador',
        'sanangelo', 'sanantonio', 'sandiego', 'sandusky', 'sanmarcos', 'santafe',
        'santamaria', 'santiago', 'santodomingo', 'saopaulo', 'sapporo', 'sarasota',
        'sardinia', 'sarnia', 'saskatoon', 'savannah', 'scottsbluff', 'scranton',
        'sd', 'seattle', 'seks', 'semo', 'sendai', 'seoul',
        'sevilla', 'sfbay', 'shanghai', 'sheboygan', 'sheffield', 'shenyang',
        'shenzhen', 'sherbrooke', 'shoals', 'showlow', 'shreveport', 'sicily',
        'sierravista', 'singapore', 'siouxcity', 'siouxfalls', 'skeena', 'slo',
        'smd', 'soo', 'southbend', 'southcoast', 'southjersey', 'spacecoast',
        'spokane', 'springfield', 'springfieldil', 'statesboro', 'staugustine', 'stcloud',
        'stgeorge', 'stillwater', 'stjoseph', 'stlouis', 'stockholm', 'stockton',
        'stpetersburg', 'strasbourg', 'stuttgart', 'sudbury', 'sunshine', 'surat',
        'susanville', 'swks', 'swmi', 'swv', 'swva', 'sydney',
        'syracuse', 'taipei', 'tallahassee', 'tampa', 'tehran', 'telaviv',
        'terrehaute', 'territories', 'texarkana', 'texoma', 'thumb', 'thunderbay',
        'tijuana', 'tippecanoe', 'tokyo', 'toledo', 'topeka', 'torino',
        'toronto', 'toulouse', 'tricities', 'troisrivieres', 'tucson', 'tulsa',
        'tunis', 'tuscaloosa', 'tuscarawas', 'twinfalls', 'twintiers', 'ukraine',
        'up', 'utica', 'valdosta', 'valencia', 'vancouver', 'venice',
        'ventura', 'veracruz', 'vermont', 'victoriatx', 'vienna', 'vietnam',
        'virgin', 'visalia', 'waco', 'warsaw', 'washingtondc', 'waterloo',
        'watertown', 'wausau', 'wellington', 'wenatchee', 'westernmass', 'westky',
        'westmd', 'westslope', 'wheeling', 'whistler', 'whitehorse', 'wichita',
        'wichitafalls', 'williamsport', 'wilmington', 'winchester', 'winnipeg', 'winstonsalem',
        'wollongong', 'worcester', 'wuhan', 'wv', 'wyoming', 'xian',
        'yakima', 'yellowknife', 'york', 'youngstown', 'yubasutter', 'yucatan',
        'yuma', 'zagreb', 'zamboanga', 'zanesville', 'zurich',
    }
    # fmt: on



"""
pycraigslist.query
~~~~~~~~~~~~~~~~~~

A suite of modules that interface with the Craigslist API.
"""

from pycraigslist.query.filters import (
    parse_filters,
    get_addl_filters,
    get_addl_filters_readable,
    validate_region,
)
from pycraigslist.query.search import fetch_posts, get_total_post_count
from pycraigslist.query.search_detail import fetch_detailed_posts



"""
pycraigslist.tests.test_api.test_exceptions
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Tests proper exception raises from pycraigslist.api.* instances.
"""

from functools import partial

import pytest
from pytest import mark

import specs

# Make a tuple of invalid pycraigslist.api.* partial instances for `test_invalid_instantiation`.
INVALID_INSTANCES = tuple(
    partial(obj, **kwargs)
    for kwargs in specs.kwargs.invalid
    for obj in specs.obj.pycraigslist_parents
)


@mark.parametrize("invalid_instance", INVALID_INSTANCES)
def test_invalid_instantiation(invalid_instance):
    """Tests invalid instantiation of pycraigslist.api.* class using invalid kwargs.
    Invalid kwargs could pertain to the following: bad site key, bad area key, bad filter key.
    All invalid instantiation should result in a ValueError."""
    with pytest.raises(ValueError):
        invalid_instance()



"""
pycraigslist.query.search_detail
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Adds details to posts from a Craigslist query.
"""

from pycraigslist.query import sessions


def fetch_detailed_posts(general_search, **kwargs):
    """Fetches additional details from every post in `general_search`."""
    # Yield all post values from general search and link with post id.
    posts = {post["id"]: post for post in general_search}
    posts_url = [post["url"] for post in posts.values()]
    params = [{} for _ in posts_url]
    # Get every post's HTML content.
    post_htmls = sessions.yield_html(posts_url, params=params)
    kwargs["ref_filters"] = invert(kwargs["ref_filters"])

    # Yield detailed posts to caller.
    for html in post_htmls:
        post_details = get_post_details(html, **kwargs)
        try:
            # Because we're dealing with asynchronous requests, link post details
            # to standard post content by post id.
            yield {**posts[post_details["id"]], **post_details}
        except KeyError:
            # If post ID in details doesn't match post IDs from general search.
            pass


def invert(ref_filters):
    """Inverts reference query filters for a faster look-up time downstream."""
    inv_filters = {}
    for key, value in ref_filters.items():
        try:
            # From {"cats_ok": {"url_key": "pets_cat", "value": 1, "attr": "cats are ok - purrr"},}
            # To {"cats are ok - purrr": {"cats_ok": "true"},}
            inv_filters[value["attr"]] = {key: "true"}
        except KeyError:
            # For filters with multiple values.
            # From {'auto_bodytype': ['bus', 'convertible', ... ],}
            # To {'bus': 'auto_bodytype', 'convertible': 'auto_bodytype', ... ,}
            if isinstance(value, dict):
                inv_filters.update({child_value: key for child_value in value})

    return inv_filters


def get_post_details(post_content, **kwargs):
    """Gets additional post details as dictionary."""
    addl_detail = {}
    addl_detail.update(get_post_id(post_content))
    addl_detail.update(get_geotag(post_content))
    addl_detail.update(get_address(post_content))
    addl_detail.update(get_post_attrs(post_content, **kwargs))
    include_body = kwargs.get("include_body")
    if include_body in (True, 1):
        addl_detail.update(get_body(post_content))
    return addl_detail


def get_post_id(post_content):
    """Gets post ID from posts's HTML content."""
    post_infos = post_content.find_all("p", {"class": "postinginfo"})
    post_attr = {"id": ""}
    for post_info in post_infos:
        if post_info.string is None:
            continue
        post_info_str = post_info.string.lower()
        if "id: " in post_info_str:
            post_attr["id"] = post_info_str.split("id: ").pop()
            break

    return post_attr


def get_geotag(post_content):
    """Gets latitude and longitude from post's HTML content."""
    post_map = post_content.find("div", {"id": "map"})
    geotag_attr = {"lat": "", "lon": ""}
    if post_map is not None:
        geotag_attr["lat"] = post_map.attrs["data-latitude"]
        geotag_attr["lon"] = post_map.attrs["data-longitude"]
    return geotag_attr


def get_address(post_content):
    """Gets address from post's HTML content."""
    post_address = post_content.find("div", {"class": "mapaddress"})
    address_attr = {"address": ""}
    if post_address is not None:
        address_attr["address"] = post_address.text
    return address_attr


def get_body(post_content):
    """Gets body from post's HTML content."""
    post_body = post_content.find("section", {"id": "postingbody"})
    body_attr = {"body": ""}
    if post_body is not None:
        # Remove elements with null attributes.
        post_body_text = (
            getattr(element, "text", element)
            for element in post_body
            if not getattr(element, "attrs", None)
        )
        # Squelch post_body_text to str.
        body_attr["body"] = str("".join(post_body_text).strip())
    return body_attr


def get_post_attrs(post_content, **kwargs):
    """Gets additional attributes from post's HTML content."""
    ref_filters = kwargs["ref_filters"]
    # Attributes without a recognizable ID will be appended to "misc".
    post_attrs = {"misc": []}
    for attr in get_attrs(post_content):
        # Add to post_attrs !! BY REFERENCE !!.
        parse_attrs(post_attrs, attr, ref_filters)

    return post_attrs


def get_attrs(post_content):
    """Gets attribute keys from post's HTML content."""
    for attribute in post_content.find_all("p", {"class": "attrgroup"}):
        for attr in attribute.find_all("span"):
            attr_text = attr.text.strip()
            if attr_text:
                yield attr_text.lower()


def parse_attrs(post_attrs, attr, ref_filters):
    """Parses attributes using reference query filters. Adds parsed
    attributes by reference to attributes collection (post_attrs)."""
    if attr in ref_filters:
        if isinstance(ref_filters[attr], dict):
            # E.g. update with {"cats_ok": "true"} from {"cats are ok - purrr": {"cats_ok": "true"},}
            post_attrs.update(ref_filters[attr])
        else:
            # Update with attribute value.
            post_attrs.update({ref_filters[attr]: attr})
    elif ":" in attr:
        # Some attr key, value are delimited by ':' - parse it to dict and update.
        # These attr keys are commonly separated by whitespaces and backslashes - replace with '_'.
        key_ = attr.split(":")[0].strip().replace(" / ", " ").replace(" ", "_")
        attr_ = attr.split(":")[1].strip()
        post_attrs.update({key_: attr_})
    else:
        # attr not found in filter - append to list of anonymous attributes.
        post_attrs["misc"].append(attr)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pycraigslist', 'pycraigslist.query']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.18.0',
 'beautifulsoup4>=4.9.0',
 'tenacity>=7.0.0',
 'cchardet>=2.1.0',
 'lxml>=4.6.0']

setup(name='pycraigslist',
      version='0.6.3',
      description='pycraigslist API library',
      author='Ira Horecka',
      author_email='ira89@icloud.com',
      url='https://github.com/irahorecka/pycraigslist',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.6',
     )



"""
pycraigslist API library
~~~~~~~~~~~~~~~~~~~~~~~~

pycraigslist is a fast and expressive Craigslist API wrapper,
written in Python, for human beings.
"""

# fmt: off
from pycraigslist.api import (community, events, forsale, gigs,
                              housing, jobs, services, resumes)

__all__ = ["community", "events", "forsale", "gigs",
           "housing", "jobs", "services", "resumes"]
# fmt: on

__version__ = "0.6.3"



"""
pycraigslist.base
~~~~~~~~~~~~~~~~~

Handles communication between the User and Craigslist.
"""

import urllib
from pycraigslist import filters, query
from pycraigslist.utils import parse_limit


class BaseAPI:
    """Base class for interfacing with the Craigslist API."""

    category = ""
    # Standard query filters that are permitted within category.
    query_filters = {}

    def __init__(self, site="sfbay", area="", filters=None, **kwargs):
        # Validate site and area values from caller.
        query.validate_region(site, area)
        self.site = site
        self.area = area
        # Get additional filters joined with standard filters.
        self.all_query_filters = {
            **self.query_filters,
            **query.get_addl_filters(self._base_url),
        }
        # Get parsed filters to use as HTTP parameters.
        self.filters = query.parse_filters(filters, self.all_query_filters, **kwargs)

    @parse_limit
    def search(self, limit=None):
        """Yields Craigslist posts as dictionary."""
        yield from query.fetch_posts(
            self._base_url, self.filters, category=self.category, limit=limit
        )

    @parse_limit
    def search_detail(self, limit=None, include_body=False):
        """Yields detailed Craigslist posts as dictionary."""
        yield from query.fetch_detailed_posts(
            self.search(limit=limit),
            include_body=include_body,
            ref_filters=self.all_query_filters,
        )

    @property
    def count(self):
        """Gets approximate number of Craigslist posts."""
        return query.get_total_post_count(self._base_url, self.filters)

    @property
    def url(self):
        """Builds full Craigslist query URL."""
        return "%s?%s" % (
            self._base_url,
            urllib.parse.urlencode(self.filters, doseq=True),
        )

    @property
    def _base_url(self):
        """Builds standard Craigslist query URL."""
        if not self.area:
            return "https://%s.craigslist.org/search/%s" % (self.site, self.category)
        return "https://%s.craigslist.org/search/%s/%s" % (
            self.site,
            self.area,
            self.category,
        )

    def get_filters(self):
        """Gets Craigslist query filters in a readable format."""
        readable_filters = {
            key: "..." if value["value"] is None else "True/False"
            for key, value in self.query_filters.items()
        }
        return {**readable_filters, **query.get_addl_filters_readable(self._base_url)}


class ParentMethods:
    """Provides parent classes with additional class methods."""

    @classmethod
    def get_categories(cls):
        """Gets valid Craigslist categories of instance."""
        return filters.category.get(cls.__name__)



"""
pycraigslist.query.sessions
~~~~~~~~~~~~~~~~~~~~~~~~~~~

Handles requests and constructs BeautifulSoup objects.
"""

import concurrent.futures

# Keep cchardet imported - operates in the background with lxml.
import cchardet
import lxml
import tenacity
import httpx
from bs4 import BeautifulSoup, SoupStrainer

from pycraigslist.exceptions import MaximumRequestsError

HEADERS = {"headers": {"User-Agent": "Mozilla/5.0"}}
# Retry 12 times, starting with 0.01 second and doubling the delay every time.
_RETRY_ARGS = {
    "wait": tenacity.wait.wait_random_exponential(multiplier=0.01, exp_base=2),
    "stop": tenacity.stop.stop_after_attempt(12),
}


def yield_html(url, **kwargs):
    """Yields HTML content(s) to caller."""
    session = httpx.Client()
    strainer = get_cl_strainer()
    try:
        # Single request: a URL string
        if isinstance(url, str):
            yield get_html(get_request(session, url, **parse_kwargs(kwargs)).text, strainer)
        # Single request: a single URL in a list or tuple
        elif isinstance(url, (list, tuple)) and len(url) == 1:
            yield get_html(get_request(session, url[0], **parse_kwargs(kwargs)).text, strainer)
        # Multiple requests
        else:
            # Build iterables of session and strainer objects equal in length to URL tuple.
            sessions = make_iterable(session, len(url))
            strainers = make_iterable(strainer, len(url))
            yield from map(
                get_html,
                (
                    response.text
                    for response in threaded_get_request(sessions, url, **parse_kwargs(kwargs))
                ),
                strainers,
            )
    except tenacity.RetryError:
        raise MaximumRequestsError("Maximum requests attempted - check network connection.")


def get_cl_strainer():
    """Gets bs4.SoupStrainer object, targeting relevant sections of the Craigslist page."""

    def target_elem_attrs(elem, attrs):
        """Gets desired elements and attributes from Craigslist HTML document."""
        if elem == "script":
            return True
        if (elem, attrs.get("class")) in (
            ("section", "userbody"),
            ("div", "search-attribute "),  # Trailing whitespace necessary
            ("div", "search-attribute hide-list"),
            ("span", "totalcount"),
            ("ul", "rows"),
        ):
            return True

    return SoupStrainer(target_elem_attrs)


def get_html(text, strainer):
    """Gets bs4.BeautifulSoup object from response text."""
    return BeautifulSoup(text, "lxml", parse_only=strainer)


@tenacity.retry(**_RETRY_ARGS)
def get_request(requests_session, url, params=None):
    """Gets httpx.Response object using httpx.Client.get.
    Retry request if request fails, with number of attempts and
    wait time specified in _RETRY_ARGS."""
    if params is None:
        params = HEADERS
    # Don't add headers if params is {}.
    elif params != {}:
        params.update(HEADERS)
    return requests_session.get(url, params=params, timeout=5)


def threaded_get_request(sessions, urls, **kwargs):
    """Yields requests from get_request concurrently."""
    yield from iter(
        concurrency(
            concurrent.futures.ThreadPoolExecutor,
            lambda args: get_request(*args),
            sessions,
            urls,
            **kwargs
        )
    )


def concurrency(PoolExecutor, map_func, *args, **kwargs):
    """General concurrency procedure for thread pools and process
    pools that submits map-able functions to arguments."""
    # Zip args and kwarg values to make tuple of args.
    zipped_args = zip(*args, *kwargs.values())
    with PoolExecutor(max_workers=5) as executor:
        futures = {
            # Func must accept all positional arguments (kwargs assumed by args OK).
            executor.submit(map_func, arg_tuple)
            for arg_tuple in zipped_args
        }
        for future in concurrent.futures.as_completed(futures):
            yield future.result()


def parse_kwargs(kwargs):
    """Removes first kwarg value from list or tuple wrapper if wrapped and
    is the only value."""
    for key, value in kwargs.copy().items():
        if isinstance(value, (list, tuple)) and len(value) == 1:
            kwargs[key] = value[0]
    return kwargs


def make_iterable(target, count):
    """Returns iterable of target object equal in length to count."""
    return (target for _ in range(count))



"""
pycraigslist.utils
~~~~~~~~~~~~~~~~~~

Provides utility functions for pycraigslist.base.
"""

from functools import wraps


def parse_limit(method):
    """Wrapper to parse 'limit' kwarg in base.BaseAPI.search
    and base.BaseAPI.search_detail methods."""

    @wraps(method)
    def wrapper(*args, **kwargs):
        limit = kwargs.get("limit")
        # Yield method if limit is valid.
        if limit is None or isinstance(limit, int) and limit >= 0:
            yield from method(*args, **kwargs)
        elif not isinstance(limit, int):
            raise TypeError("'limit' must be of type 'int'")
        else:
            raise ValueError("'limit' must be greater than / equal to 0")

    return wrapper
